@@ -19,7 +19,6 @@
  */
 
 #define COBJMACROS
-#define NONAMELESSUNION
 
 #include "dwrite_private.h"
 #include "winternl.h"
@@ -114,7 +113,8 @@ enum OPENTYPE_CMAP_TABLE_FORMAT
 #include "pshpack2.h"
 typedef struct
 {
-    ULONG version;
+    USHORT majorVersion;
+    USHORT minorVersion;
     ULONG revision;
     ULONG checksumadj;
     ULONG magic;
@@ -205,7 +205,8 @@ typedef struct
 } TT_OS2_V2;
 
 typedef struct {
-    ULONG  version;
+    USHORT majorVersion;
+    USHORT minorVersion;
     SHORT  ascender;
     SHORT  descender;
     SHORT  linegap;
@@ -1861,10 +1862,10 @@ HRESULT opentype_get_cpal_entries(const void *cpal, UINT32 palette, UINT32 first
     first_entry_index += GET_BE_WORD(header->colorRecordIndices[palette]);
 
     for (i = 0; i < entry_count; i++) {
-        entries[i].u1.r = records[first_entry_index + i].red   / 255.0f;
-        entries[i].u2.g = records[first_entry_index + i].green / 255.0f;
-        entries[i].u3.b = records[first_entry_index + i].blue  / 255.0f;
-        entries[i].u4.a = records[first_entry_index + i].alpha / 255.0f;
+        entries[i].r = records[first_entry_index + i].red   / 255.0f;
+        entries[i].g = records[first_entry_index + i].green / 255.0f;
+        entries[i].b = records[first_entry_index + i].blue  / 255.0f;
+        entries[i].a = records[first_entry_index + i].alpha / 255.0f;
     }
 
     return S_OK;
