@@ -1346,23 +1346,6 @@ void __cdecl __ExceptionPtrCopy(exception_ptr *ep, const exception_ptr *copy)
         InterlockedIncrement(copy->ref);
 }
 
-/*********************************************************************
- * ?__ExceptionPtrAssign@@YAXPAXPBX@Z
- * ?__ExceptionPtrAssign@@YAXPEAXPEBX@Z
- */
-void __cdecl __ExceptionPtrAssign(exception_ptr *ep, const exception_ptr *assign)
-{
-    TRACE("(%p %p)\n", ep, assign);
-
-    /* don't destroy object stored in ep */
-    if (ep->ref)
-        InterlockedDecrement(ep->ref);
-
-    *ep = *assign;
-    if (ep->ref)
-        InterlockedIncrement(ep->ref);
-}
-
 /*********************************************************************
  * ?__ExceptionPtrRethrow@@YAXPBX@Z
  * ?__ExceptionPtrRethrow@@YAXPEBX@Z
@@ -1597,11 +1580,6 @@ void __cdecl __ExceptionPtrCopyException(exception_ptr *ep,
 }
 #endif
 
-MSVCRT_bool __cdecl __ExceptionPtrCompare(const exception_ptr *ep1, const exception_ptr *ep2)
-{
-    return ep1->rec == ep2->rec;
-}
-
 void* __cdecl __AdjustPointer(void *obj, const this_ptr_offsets *off)
 {
     return get_this_pointer(off, obj);
