@@ -749,7 +749,7 @@ static int puts_clbk_str_c99_a(void *ctx, int len, const char *str)
 }
 
 /*********************************************************************
- *              __stdio_common_vsprintf (UCRTBASE.@)
+ *		__stdio_common_vsprintf (MSVCRT.@)
  */
 int CDECL MSVCRT__stdio_common_vsprintf( unsigned __int64 options, char *str, MSVCRT_size_t len, const char *format,
                                          MSVCRT__locale_t locale, __ms_va_list valist )
@@ -918,7 +918,7 @@ int CDECL MSVCRT_vsnprintf_s( char *str, MSVCRT_size_t sizeOfBuffer,
 }
 
 /*********************************************************************
- *              __stdio_common_vsnprintf_s (UCRTBASE.@)
+ *		__stdio_common_vsnprintf_s (MSVCRT.@)
  */
 int CDECL MSVCRT__stdio_common_vsnprintf_s( unsigned __int64 options,
         char *str, MSVCRT_size_t sizeOfBuffer, MSVCRT_size_t count,
@@ -930,7 +930,7 @@ int CDECL MSVCRT__stdio_common_vsnprintf_s( unsigned __int64 options,
 }
 
 /*********************************************************************
- *              __stdio_common_vsnwprintf_s (UCRTBASE.@)
+ *		__stdio_common_vsnwprintf_s (MSVCRT.@)
  */
 int CDECL MSVCRT__stdio_common_vsnwprintf_s( unsigned __int64 options,
         MSVCRT_wchar_t *str, MSVCRT_size_t sizeOfBuffer, MSVCRT_size_t count,
@@ -942,7 +942,7 @@ int CDECL MSVCRT__stdio_common_vsnwprintf_s( unsigned __int64 options,
 }
 
 /*********************************************************************
- *              __stdio_common_vswprintf_s (UCRTBASE.@)
+ *		__stdio_common_vswprintf_s (MSVCRT.@)
  */
 int CDECL MSVCRT__stdio_common_vswprintf_s( unsigned __int64 options,
         MSVCRT_wchar_t *str, MSVCRT_size_t count, const MSVCRT_wchar_t *format,
@@ -952,7 +952,7 @@ int CDECL MSVCRT__stdio_common_vswprintf_s( unsigned __int64 options,
 }
 
 /*********************************************************************
- *              __stdio_common_vsprintf_s (UCRTBASE.@)
+ *		__stdio_common_vsprintf_s (MSVCRT.@)
  */
 int CDECL MSVCRT__stdio_common_vsprintf_s( unsigned __int64 options,
         char *str, MSVCRT_size_t count, const char *format,
@@ -1159,7 +1159,7 @@ int CDECL MSVCRT__vswprintf_p(MSVCRT_wchar_t *buffer, MSVCRT_size_t length,
 
 
 /*********************************************************************
- *              __stdio_common_vswprintf_p (UCRTBASE.@)
+ *		__stdio_common_vswprintf_p (MSVCRT.@)
  */
 int CDECL MSVCRT__stdio_common_vswprintf_p( unsigned __int64 options,
         MSVCRT_wchar_t *str, MSVCRT_size_t count, const MSVCRT_wchar_t *format,
@@ -1266,7 +1266,7 @@ static int puts_clbk_str_c99_w(void *ctx, int len, const MSVCRT_wchar_t *str)
 }
 
 /*********************************************************************
- *              __stdio_common_vswprintf (UCRTBASE.@)
+ *		__stdio_common_vswprintf (MSVCRT.@)
  */
 int CDECL MSVCRT__stdio_common_vswprintf( unsigned __int64 options,
         MSVCRT_wchar_t *str, MSVCRT_size_t len, const MSVCRT_wchar_t *format,
@@ -2015,6 +2015,36 @@ INT CDECL MSVCRT_wcsncat_s(MSVCRT_wchar_t *dst, MSVCRT_size_t elem,
     return MSVCRT_ERANGE;
 }
 
+/*********************************************************************
+ * wctoint (INTERNAL)
+ */
+static int wctoint(WCHAR c, int base)
+{
+    int v = -1;
+    if ('0' <= c && c <= '9')
+        v = c - '0';
+    else if ('A' <= c && c <= 'Z')
+        v = c - 'A' + 10;
+    else if ('a' <= c && c <= 'z')
+        v = c - 'a' + 10;
+    else {
+        /* NOTE: wine_fold_string(MAP_FOLDDIGITS) supports too many things. */
+        /* Unicode points that contain digits 0-9; keep this sorted! */
+        static const WCHAR zeros[] = {
+            0x660, 0x6f0, 0x966, 0x9e6, 0xa66, 0xae6, 0xb66, 0xc66, 0xce6,
+            0xd66, 0xe50, 0xed0, 0xf20, 0x1040, 0x17e0, 0x1810, 0xff10
+        };
+        int i;
+        for (i = 0; i < sizeof(zeros)/sizeof(zeros[0]) && c >= zeros[i]; ++i) {
+            if (zeros[i] <= c && c <= zeros[i] + 9) {
+                v = c - zeros[i];
+                break;
+            }
+        }
+    }
+    return v < base ? v : -1;
+}
+
 /*********************************************************************
  *  _wcstoi64_l (MSVCRT.@)
  *
@@ -2023,7 +2053,7 @@ INT CDECL MSVCRT_wcsncat_s(MSVCRT_wchar_t *dst, MSVCRT_size_t elem,
 __int64 CDECL MSVCRT__wcstoi64_l(const MSVCRT_wchar_t *nptr,
         MSVCRT_wchar_t **endptr, int base, MSVCRT__locale_t locale)
 {
-    BOOL negative = FALSE;
+    BOOL negative = FALSE, empty = TRUE;
     __int64 ret = 0;
 
     TRACE("(%s %p %d %p)\n", debugstr_w(nptr), endptr, base, locale);
@@ -2032,6 +2062,9 @@ __int64 CDECL MSVCRT__wcstoi64_l(const MSVCRT_wchar_t *nptr,
     if (!MSVCRT_CHECK_PMT(base == 0 || base >= 2)) return 0;
     if (!MSVCRT_CHECK_PMT(base <= 36)) return 0;
 
+    if(endptr)
+        *endptr = (MSVCRT_wchar_t*)nptr;
+
     while(isspaceW(*nptr)) nptr++;
 
     if(*nptr == '-') {
@@ -2040,36 +2073,28 @@ __int64 CDECL MSVCRT__wcstoi64_l(const MSVCRT_wchar_t *nptr,
     } else if(*nptr == '+')
         nptr++;
 
-    if((base==0 || base==16) && *nptr=='0' && tolowerW(*(nptr+1))=='x') {
+    if((base==0 || base==16) && wctoint(*nptr, 1)==0 && tolowerW(*(nptr+1))=='x') {
         base = 16;
         nptr += 2;
     }
 
     if(base == 0) {
-        if(*nptr=='0')
+        if(wctoint(*nptr, 1)==0)
             base = 8;
         else
             base = 10;
     }
 
     while(*nptr) {
-        MSVCRT_wchar_t cur = tolowerW(*nptr);
-        int v;
-
-        if(cur>='0' && cur<='9') {
-            if(cur >= '0'+base)
-                break;
-            v = cur-'0';
-        } else {
-            if(cur<'a' || cur>='a'+base-10)
-                break;
-            v = cur-'a'+10;
-        }
+        int v = wctoint(*nptr, base);
+        if(v<0)
+            break;
 
         if(negative)
             v = -v;
 
         nptr++;
+        empty = FALSE;
 
         if(!negative && (ret>MSVCRT_I64_MAX/base || ret*base>MSVCRT_I64_MAX-v)) {
             ret = MSVCRT_I64_MAX;
@@ -2081,7 +2106,7 @@ __int64 CDECL MSVCRT__wcstoi64_l(const MSVCRT_wchar_t *nptr,
             ret = ret*base + v;
     }
 
-    if(endptr)
+    if(endptr && !empty)
         *endptr = (MSVCRT_wchar_t*)nptr;
 
     return ret;
@@ -2188,7 +2213,7 @@ MSVCRT_longlong __cdecl MSVCRT__wtoll(const MSVCRT_wchar_t *str)
 unsigned __int64 CDECL MSVCRT__wcstoui64_l(const MSVCRT_wchar_t *nptr,
         MSVCRT_wchar_t **endptr, int base, MSVCRT__locale_t locale)
 {
-    BOOL negative = FALSE;
+    BOOL negative = FALSE, empty = TRUE;
     unsigned __int64 ret = 0;
 
     TRACE("(%s %p %d %p)\n", debugstr_w(nptr), endptr, base, locale);
@@ -2197,6 +2222,9 @@ unsigned __int64 CDECL MSVCRT__wcstoui64_l(const MSVCRT_wchar_t *nptr,
     if (!MSVCRT_CHECK_PMT(base == 0 || base >= 2)) return 0;
     if (!MSVCRT_CHECK_PMT(base <= 36)) return 0;
 
+    if(endptr)
+        *endptr = (MSVCRT_wchar_t*)nptr;
+
     while(isspaceW(*nptr)) nptr++;
 
     if(*nptr == '-') {
@@ -2205,33 +2233,25 @@ unsigned __int64 CDECL MSVCRT__wcstoui64_l(const MSVCRT_wchar_t *nptr,
     } else if(*nptr == '+')
         nptr++;
 
-    if((base==0 || base==16) && *nptr=='0' && tolowerW(*(nptr+1))=='x') {
+    if((base==0 || base==16) && wctoint(*nptr, 1)==0 && tolowerW(*(nptr+1))=='x') {
         base = 16;
         nptr += 2;
     }
 
     if(base == 0) {
-        if(*nptr=='0')
+        if(wctoint(*nptr, 1)==0)
             base = 8;
         else
             base = 10;
     }
 
     while(*nptr) {
-        MSVCRT_wchar_t cur = tolowerW(*nptr);
-        int v;
-
-        if(cur>='0' && cur<='9') {
-            if(cur >= '0'+base)
-                break;
-            v = *nptr-'0';
-        } else {
-            if(cur<'a' || cur>='a'+base-10)
-                break;
-            v = cur-'a'+10;
-        }
+        int v = wctoint(*nptr, base);
+        if(v<0)
+            break;
 
         nptr++;
+        empty = FALSE;
 
         if(ret>MSVCRT_UI64_MAX/base || ret*base>MSVCRT_UI64_MAX-v) {
             ret = MSVCRT_UI64_MAX;
@@ -2240,7 +2260,7 @@ unsigned __int64 CDECL MSVCRT__wcstoui64_l(const MSVCRT_wchar_t *nptr,
             ret = ret*base + v;
     }
 
-    if(endptr)
+    if(endptr && !empty)
         *endptr = (MSVCRT_wchar_t*)nptr;
 
     return negative ? -ret : ret;
