@@ -80,11 +80,14 @@ enum wined3d_gl_extension
     ARB_POINT_SPRITE,
     ARB_PROVOKING_VERTEX,
     ARB_SAMPLER_OBJECTS,
+    ARB_SHADER_ATOMIC_COUNTERS,
     ARB_SHADER_BIT_ENCODING,
     ARB_SHADER_IMAGE_LOAD_STORE,
     ARB_SHADER_IMAGE_SIZE,
+    ARB_SHADER_STORAGE_BUFFER_OBJECT,
     ARB_SHADER_TEXTURE_LOD,
     ARB_SHADING_LANGUAGE_100,
+    ARB_SHADING_LANGUAGE_420PACK,
     ARB_SHADING_LANGUAGE_PACKING,
     ARB_SHADOW,
     ARB_STENCIL_TEXTURING,
